/*
 *   Copyright (C) 2001  Andrea Gini
 *
 *   Require JDK 1.3
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Send questions or suggestions to ginian@tiscalinet.it
 *
 */
import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class TextEditor extends JFrame implements ActionListener {

  private JTextComponent editor;
  private JFileChooser fileChooser;

  protected JMenuItem OpenMenuItem;
  protected JMenuItem SaveMenuItem;
  protected JMenuItem CutMenuItem;
  protected JMenuItem CopyMenuItem;
  protected JMenuItem PasteMenuItem; 
    
  protected AbstractButton OpenButton; 
  protected AbstractButton SaveButton;
  protected AbstractButton CutButton;
  protected AbstractButton CopyButton;
  protected AbstractButton PasteButton;

  public TextEditor() {
    super();
    setupFrame();
    setupFrameBehaviour();
    setupComponents();
    buildGui();
    registerListeners();
  }
  protected void setupFrame() {
    setTitle("TextEditor");
    setSize(300,300);
  }
  protected void setupFrameBehaviour() {
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);  
  }
  protected void setupComponents() {
    editor = createEditor();
    fileChooser = createFileChooser();
    buildButtons();
    buildMenuItems();
  }
  protected JTextComponent createEditor() {
    return new JTextArea();
  }
  protected JFileChooser createFileChooser() {
    return new JFileChooser();
  }
  public JTextComponent getEditor() {
    return editor;
  }
  protected JFileChooser getFileChooser() {
    return fileChooser;
  }
  protected void buildMenuItems() {
    OpenMenuItem = createMenuItem("Open",new ImageIcon("Open24.gif"));
    SaveMenuItem = createMenuItem("Save",new ImageIcon("Save24.gif"));
    CutMenuItem = createMenuItem("Cut",new ImageIcon("Cut24.gif"));
    CopyMenuItem = createMenuItem("Copy",new ImageIcon("Copy24.gif"));
    PasteMenuItem = createMenuItem("Paste",new ImageIcon("Paste24.gif"));
  }
  protected JMenuItem createMenuItem(String name,Icon icon) {
    return new JMenuItem(name,icon);
  }
  protected JMenuItem createMenuItem(String name) {
    return new JMenuItem(name);
  }
  protected void buildButtons() {
    OpenButton = createToolbarButton(new ImageIcon("Open24.gif"));
    SaveButton = createToolbarButton(new ImageIcon("Save24.gif"));
    CutButton = createToolbarButton(new ImageIcon("Cut24.gif"));
    CopyButton = createToolbarButton(new ImageIcon("Copy24.gif"));
    PasteButton = createToolbarButton(new ImageIcon("Paste24.gif"));
  }
  protected AbstractButton createToolbarButton(Icon icon) {
    return new JButton(icon);
  }
  protected void buildGui() {
    JMenuBar menuBar = createMenuBar();
    JToolBar toolBar = createToolBar();

    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(BorderLayout.NORTH,toolBar);
    getContentPane().add(BorderLayout.CENTER,new JScrollPane(getEditor()));
    setJMenuBar(menuBar);
  }
  protected JMenuBar createMenuBar() {
    JMenu menu = new JMenu("Menu");
      menu.add(OpenMenuItem);
      menu.add(SaveMenuItem);
      menu.addSeparator();
      menu.add(CutMenuItem);
      menu.add(CopyMenuItem);
      menu.add(PasteMenuItem);
    JMenuBar menuBar = new JMenuBar();
    menuBar.add(menu);
    
    return menuBar;
  }
  protected JToolBar createToolBar() {
    JToolBar toolBar = new JToolBar();
    toolBar.add(OpenButton);
    toolBar.add(SaveButton);
    toolBar.addSeparator();
    toolBar.add(CutButton);
    toolBar.add(CopyButton);
    toolBar.add(PasteButton);  
    return toolBar;
  }
  protected void registerListeners() {
    OpenMenuItem.addActionListener(this);
    SaveMenuItem.addActionListener(this);
    CutMenuItem.addActionListener(this);
    CopyMenuItem.addActionListener(this);
    PasteMenuItem.addActionListener(this);

    OpenButton.addActionListener(this);
    SaveButton.addActionListener(this);
    CutButton.addActionListener(this);
    CopyButton.addActionListener(this);
    PasteButton.addActionListener(this);
  }
  protected void open() {
      int response = getFileChooser().showOpenDialog(this);
      if(response==JFileChooser.APPROVE_OPTION) {
        try {
          File f = getFileChooser().getSelectedFile();
          Reader in = new FileReader(f);
          editor.read(in,null);
        }
        catch(Exception e) {}
      }  
  }
  protected void save() {
    int response = getFileChooser().showSaveDialog(this);
    if(response==JFileChooser.APPROVE_OPTION) {
      try {
        File f = getFileChooser().getSelectedFile();
        Writer out = new FileWriter(f);
        editor.write(out);
      }
      catch(Exception e) {}
    }  
  }
  protected void cut() {
    getEditor().cut();
  }
  protected void copy() {
    getEditor().copy();
  }
  protected void paste() {
    getEditor().paste();
  }
  
  public void actionPerformed(ActionEvent ae) {
    if(ae.getSource().equals(OpenButton) || ae.getSource().equals(OpenMenuItem)) {
      open();
    }
    else if(ae.getSource().equals(SaveButton) || ae.getSource().equals(SaveMenuItem)) {
      save();
    }
    else if(ae.getSource().equals(CutButton) || ae.getSource().equals(CutMenuItem)) {
      cut();
    }
    else if(ae.getSource().equals(CopyButton) || ae.getSource().equals(CopyMenuItem)) {
      copy();
    }
    else if(ae.getSource().equals(PasteButton) || ae.getSource().equals(PasteMenuItem)) {
      paste();
    }      
  }
  
  public static void main(String argv[]) {
    TextEditor t = new TextEditor();
    t.setVisible(true);
  }
}